// wdjsrvc2.c

#include <windows.h>
#include <stdio.h>
#include "wdjsrvc.h"

HANDLE hDoneEvent = NULL, hThread = NULL;
DWORD dwCurrentState;
SERVICE_STATUS_HANDLE hService;

BOOL NotifySCM(DWORD, DWORD, DWORD);
VOID WINAPI ServiceMain(DWORD, LPTSTR *);
VOID WINAPI ServiceHandler(DWORD);
DWORD WINAPI WorkerThread(LPVOID);

//-----------------------------------------------------------------
void main(int argc, char **argv)
{
    SERVICE_TABLE_ENTRY ServiceTable[] = {
        {WDJSRVC_NAME, ServiceMain}, {NULL, NULL}
    };
    StartServiceCtrlDispatcher(ServiceTable);
}

//-----------------------------------------------------------------
VOID WINAPI ServiceMain(DWORD dwArgc, LPTSTR *lpszArgv)
{
    DWORD   ThreadId;

    if (!(hService = RegisterServiceCtrlHandler(
                           WDJSRVC_NAME, ServiceHandler))) {
        return;
    }

    NotifySCM(SERVICE_START_PENDING, 0 , 1);

    hDoneEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    hThread = CreateThread(0,0,WorkerThread,0,0,&ThreadId);

    NotifySCM(SERVICE_RUNNING, 0 , 0);
    WaitForSingleObject(hDoneEvent, INFINITE);

    CloseHandle(hThread);
    CloseHandle(hDoneEvent);
    return;
}

//-----------------------------------------------------------------
VOID WINAPI ServiceHandler(DWORD fdwControl)
{
    switch(fdwControl) {
    case SERVICE_CONTROL_STOP:
        NotifySCM(SERVICE_STOP_PENDING, 0 , 1);
        SetEvent(hDoneEvent);
        NotifySCM(SERVICE_STOPPED, 0 , 0);
        break;

    case SERVICE_CONTROL_INTERROGATE:
        NotifySCM(dwCurrentState, 0 , 0);
        break;
   
    case SERVICE_CONTROL_SHUTDOWN:
        break;

    case WDJSRVC_CONTROL_LOGON:
        MessageBox(NULL,TEXT("WDJSRVC_CONTROL_LOGON"),WDJSRVC_NAME,
                   MB_OK|MB_SERVICE_NOTIFICATION);
        break;

    case WDJSRVC_CONTROL_LOGOFF:       
        MessageBox(NULL,TEXT("WDJSRVC_CONTROL_LOGOFF"),WDJSRVC_NAME,
                   MB_OK|MB_SERVICE_NOTIFICATION);
        break;
    }
}

//-----------------------------------------------------------------
BOOL NotifySCM(DWORD State, DWORD ExitCode, DWORD Progress)
{
    SERVICE_STATUS ServiceStatus;

    ServiceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
    ServiceStatus.dwCurrentState = dwCurrentState = State;
    ServiceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP |
                                       SERVICE_ACCEPT_SHUTDOWN;
    ServiceStatus.dwWin32ExitCode = ExitCode;
    ServiceStatus.dwServiceSpecificExitCode = 0;
    ServiceStatus.dwCheckPoint = Progress;
    ServiceStatus.dwWaitHint = 5000;

    return SetServiceStatus(hService, &ServiceStatus);
}

//-----------------------------------------------------------------
DWORD WINAPI WorkerThread(LPDWORD ThreadParam)
{
    DWORD status; 
    SECURITY_ATTRIBUTES sa;
    SECURITY_DESCRIPTOR sd;
    HANDLE hEvents[2];
   
    // Just give "everyone" access by using a NULL dacl

    InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
    SetSecurityDescriptorDacl(&sd,TRUE,NULL,FALSE);
   
    sa.nLength = sizeof(SECURITY_ATTRIBUTES);
    sa.lpSecurityDescriptor = &sd;
    sa.bInheritHandle = FALSE; 

    // create events that companion "runapp" will signal

    hEvents[0] = CreateEvent(&sa,FALSE,FALSE,WDJSRVC_LOGON_EVENT); 
    hEvents[1] = CreateEvent(&sa,FALSE,FALSE,WDJSRVC_LOGOFF_EVENT);
   
    while (TRUE) {                                   
        status = WaitForMultipleObjects(2,hEvents,FALSE,INFINITE);
   
        if (status == WAIT_OBJECT_0) {
            MessageBox(NULL,TEXT("RunApp signalled a LogOn"),
                       WDJSRVC_NAME,MB_OK|MB_SERVICE_NOTIFICATION);
        } else if (status == WAIT_OBJECT_0 + 1) {
            MessageBox(NULL,TEXT("RunApp signalled a LogOff"),
                       WDJSRVC_NAME,MB_OK|MB_SERVICE_NOTIFICATION);
        }
    }
    
    return TRUE;
}

